// ===========================================================================
//
// Test Drive 7 : FrontEndPCF.java
//
// ===========================================================================
// MQSeries Java application (Alan Hollingshead)
//
// Defines MQSeries objects on the Front-End system using PCF.
// Definitions are based on the shortname chosen by the end-user (e.g. HOLLINGS)
// The shortname is used as the local Queue Manager
//
// Note: sRegSysHostname must be the hostname or IP address of the Registration System workstation (e.g. REGSYS)
//
// Command server must be active to accept PCF requests 
// "strmqcsv <QMgrName>" starts the command server
//
// Called by SupplyShortName.java
//

import com.ibm.mqbind.*;                                             // include the MQ bindings package
import com.ibm.mqbind.pcf.*;                                         // include MQ PCF package ... requires SupportPac
import java.io.*;
import java.util.*;

public class FrontEndPCF 
{
   static MQMessage []   responses;
   static MQCFH          cfh;
   static PCFParameter   p;

   
   public static void DefineFrontEndDefs(String sShortname,          // shortname used as local Queue Manager
                                         String sRegSysQMgr,
                                         String sRegSysAddress)                                         
   {                                       
      try
      {   
         PCFAgent agent;
      
         // Connect a PCFAgent to the specified queue manager
         System.out.println("\nConnecting to queue manager " + sShortname + "... ");
         agent = new PCFAgent (sShortname);
         
         System.out.println ("Sending PCF requests ... ");

         // --------------------------------------------------------------------------------------------------------------         
         // Define Remote Queue to Registration System system
         // --------------------------------------------------------------------------------------------------------------         
         String sRemoteQtoRegSys = sShortname + ".REMOTE";

         PCFParameter[] qremoteParameters =
                        {
                           new MQCFST (CMQC.MQCA_Q_NAME, sRemoteQtoRegSys), 
                           new MQCFIN (CMQC.MQIA_Q_TYPE, CMQC.MQQT_REMOTE),            // qtype = remote
                           new MQCFST (CMQC.MQCA_REMOTE_Q_NAME, sRemoteQtoRegSys),
                           new MQCFST (CMQC.MQCA_XMIT_Q_NAME, sRegSysQMgr),            // xmitq name = Front-End QMgr name
                           new MQCFST (CMQC.MQCA_REMOTE_Q_MGR_NAME, sRegSysQMgr),      // Reg Sys QMgr name
                           new MQCFIN (CMQCFC.MQIACF_REPLACE, CMQCFC.MQRP_YES)         // replace any existing defn
                        };           

         System.out.print ("   Define QREMOTE           '"+sRemoteQtoRegSys+"'... ");
         
         // Use the agent to send the request
         responses = agent.send (CMQCFC.MQCMD_CREATE_Q, qremoteParameters);

         // check responses
         checkResponses(); 

         // --------------------------------------------------------------------------------------------------------------
         // Define Alias Queue from MQSI to RegSys
         // --------------------------------------------------------------------------------------------------------------         
         String sAliasQtoRegSys = "MQSI_OUT2SAP";
         String sQtoRegSys = sShortname + ".REMOTE";
         
         PCFParameter[] pcfAliasQtoRegSysParameters =
                        {
                           new MQCFST (CMQC.MQCA_Q_NAME, sAliasQtoRegSys), 
                           new MQCFIN (CMQC.MQIA_Q_TYPE, CMQC.MQQT_ALIAS),             // qtype = alias
                           new MQCFST (CMQC.MQCA_BASE_Q_NAME, sQtoRegSys),
                           new MQCFIN (CMQCFC.MQIACF_REPLACE, CMQCFC.MQRP_YES)         // replace any existing defn
                        };           

         System.out.print ("   Define QALIAS from SAP  '"+sAliasQtoRegSys+"'... ");
         
         // Use the agent to send the request
         responses = agent.send (CMQCFC.MQCMD_CREATE_Q, pcfAliasQtoRegSysParameters);

         // check responses
         checkResponses();               

 

         // --------------------------------------------------------------------------------------------------------------
         // Define Alias Queue from RegSys to MQSI 
         // --------------------------------------------------------------------------------------------------------------         
         String sAliasQfromRegSys = sShortname + ".LOCAL";
         String sQtoMQSI = "MQSI_IN"; 
         
         PCFParameter[] pcfAliasQfromRegSysParameters =
                        {
                           new MQCFST (CMQC.MQCA_Q_NAME, sAliasQfromRegSys), 
                           new MQCFIN (CMQC.MQIA_Q_TYPE, CMQC.MQQT_ALIAS),             // qtype = alias
                           new MQCFST (CMQC.MQCA_BASE_Q_NAME, sQtoMQSI),
                           new MQCFIN (CMQCFC.MQIACF_REPLACE, CMQCFC.MQRP_YES)         // replace any existing defn
                        };           

         System.out.print ("   Define QALIAS from RegSys  '"+sAliasQfromRegSys+"'... ");
         
         // Use the agent to send the request
         responses = agent.send (CMQCFC.MQCMD_CREATE_Q, pcfAliasQfromRegSysParameters);

         // check responses
         checkResponses();               

         
         
         // --------------------------------------------------------------------------------------------------------------         
         // Define Requester Channel
         // --------------------------------------------------------------------------------------------------------------         
         String sRequesterChannelName = sRegSysQMgr + "." + sShortname;
         String sRegSysIPaddress = sRegSysAddress;   // default is "9.20.20.198";
         int hbInt = 30;
       
         PCFParameter[] requesterChannelParameters = 
                        {
                           new MQCFST (CMQCFC.MQCACH_CHANNEL_NAME, sRequesterChannelName), 
                           new MQCFIN (CMQCFC.MQIACH_CHANNEL_TYPE, CMQXC.MQCHT_REQUESTER),
                           new MQCFIN (CMQCFC.MQIACH_XMIT_PROTOCOL_TYPE, CMQXC.MQXPT_TCP),
                           new MQCFIN (CMQCFC.MQIACH_HB_INTERVAL, hbInt),
                           new MQCFST (CMQCFC.MQCACH_CONNECTION_NAME, sRegSysIPaddress)
                        };
      
         System.out.print ("   Define requester channel '"+sRequesterChannelName+"'... ");
         
         // Use the agent to send the request         
         responses = agent.send (CMQCFC.MQCMD_CREATE_CHANNEL, requesterChannelParameters);

         // check responses
         checkResponses();                           
         
      }
      catch (MQException mqe)
      {
         switch (mqe.reasonCode)
         {                                                                   
            case 2059 : System.out.println ("RC 2059. Local Queue Manager not started");
                                                                
            default   : System.out.println ("MQ exception caught:\n" + mqe);
         }   
         System.exit(0);                       
      }
      catch (IOException ioe)
      {
         System.err.println (ioe);
         System.exit(0);   
      }
   } // return to calling program  

   // ----------------------------------------------------------------------------------------------------------------
   // Method: checkResponses()
   // ----------------------------------------------------------------------------------------------------------------
   public static void checkResponses()
   {
      try
      {
         for (int i = 0; i < responses.length; i++)
         {
            cfh = new MQCFH (responses [i]);
         
            // Check the PCF header (MQCFH) in the response message

            switch (cfh.reason)
            {
               case 0    : System.out.println ("OK");      
                           break;                                                             
                                      
               case 4031 : System.out.println ("Channel in use");
                           i++;    // to ignore PCF failed message
                           break;                               
                                                                       
               case 4042 : System.out.println ("Channel already exists");
                           i++;    // to ignore PCF failed message
                           break;                               
                        
               default   : System.out.println ("PCF error:\n" + cfh);

               // Walk through the returned parameters describing the error

               for (int j = 0; j < cfh.parameterCount; j++)
               {
                  System.out.println (PCFParameter.nextParameter (responses [0]));
               }
            }
         }
      }
      
      // ----------------------------------------------------------------------------------------
      // catch exceptions
      // ----------------------------------------------------------------------------------------   
      catch (EOFException eofe)
      {
         ;
      }
      catch (IOException ioe)
      {
         System.err.println (ioe);
      }       
      catch (InstantiationException ie)
      {
         System.err.println (ie);
      }          
   }
}


                                                                                        
