// ===========================================================================
//
// Test Drive 7 : RegSysPCF.java
//
// ===========================================================================
// MQSeries Java application (Alan Hollingshead)
//
// Defines MQSeries objects on the Registration System using PCF.
// Definitions are based on the confirmed shortname and assigned logical SAP system (e.g. SYS1)
//
// Command server must be active to accept PCF requests 
// "strmqcsv <QMgrName>" starts the command server
//
//
// Called from CheckRegistration - expects <shortname> and reserved logical SAP system (e.g. SYS1)
//



import com.ibm.mqbind.*;                                             // include the MQ bindings package
import com.ibm.mqbind.pcf.*;                                         // include MQ PCF package ... requires SupportPac
import java.io.*;
import java.util.*;

public class RegSysPCF 
{
   static MQMessage []   responses;
   static MQCFH          cfh;
   static PCFParameter   p;
   static String         sRegSysQMgr = "REGSYS";
   
   public static void SetUpDefinitions(String sShortname, String sSAPsystem)
   {
      try
      {   
         PCFAgent agent;
      
         // Connect a PCFAgent to the specified queue manager
         System.out.println("\nConnecting to queue manager " + sRegSysQMgr + "... ");
         agent = new PCFAgent (sRegSysQMgr);
         
         System.out.println ("Sending PCF requests ... ");
         
         // --------------------------------------------------------------------------------------------------------------         
         // Define Remote Queue to SAP system
         // --------------------------------------------------------------------------------------------------------------         
         String sRemoteQtoSAP = sShortname + ".REMOTE";
         String sQtoSAP = "SAP.IN";
         String sSAPqmgr = "SAP";

         PCFParameter[] sRemoteQtoSAPparameters =
                        {
                           new MQCFST (CMQC.MQCA_Q_NAME, sRemoteQtoSAP), 
                           new MQCFIN (CMQC.MQIA_Q_TYPE, CMQC.MQQT_REMOTE),            // qtype = remote
                           new MQCFST (CMQC.MQCA_REMOTE_Q_NAME, sQtoSAP),
                           new MQCFST (CMQC.MQCA_XMIT_Q_NAME, sSAPqmgr),               // xmitq name = SAP QMgr name
                           new MQCFST (CMQC.MQCA_REMOTE_Q_MGR_NAME, sSAPqmgr),         // SAP QMgr name
                           new MQCFIN (CMQCFC.MQIACF_REPLACE, CMQCFC.MQRP_YES)         // replace any existing defn
                        };           

         System.out.print ("   Define QREMOTE          '"+sRemoteQtoSAP+"'... ");
         
         // Use the agent to send the request
         responses = agent.send (CMQCFC.MQCMD_CREATE_Q, sRemoteQtoSAPparameters);

         // check responses
         checkResponses(); 
  
           
         // --------------------------------------------------------------------------------------------------------------
         // Delete Alias Queue 
         // --------------------------------------------------------------------------------------------------------------         
         String sAliasQtoDelete = sSAPsystem + ".OUT";
         
         PCFParameter[] pcfAliasQtoDeleteParameters =
                        {
                           new MQCFST (CMQC.MQCA_Q_NAME, sAliasQtoDelete), 
                           new MQCFIN (CMQC.MQIA_Q_TYPE, CMQC.MQQT_ALIAS)              // qtype = alias
                        };           

         System.out.print ("   Delete QALIAS           '"+sAliasQtoDelete+"'... ");
         
         // Use the agent to send the request
         responses = agent.send (CMQCFC.MQCMD_DELETE_Q, pcfAliasQtoDeleteParameters);

         // check responses
         checkResponses();               
    
  
         
         // --------------------------------------------------------------------------------------------------------------         
         // Define Remote Queue to F/E system
         // --------------------------------------------------------------------------------------------------------------         
         String sQofFE = sShortname + ".LOCAL";
         String sRemoteQtoFE = sSAPsystem + ".OUT";

         PCFParameter[] qremoteParameters =
                        {
                           new MQCFST (CMQC.MQCA_Q_NAME, sRemoteQtoFE), 
                           new MQCFIN (CMQC.MQIA_Q_TYPE, CMQC.MQQT_REMOTE),            // qtype = remote
                           new MQCFST (CMQC.MQCA_REMOTE_Q_NAME, sQofFE),
                           new MQCFST (CMQC.MQCA_XMIT_Q_NAME, sShortname),             // xmitq name = Front-End QMgr name
                           new MQCFST (CMQC.MQCA_REMOTE_Q_MGR_NAME, sShortname),       // F/E QMgr name
                           new MQCFIN (CMQCFC.MQIACF_REPLACE, CMQCFC.MQRP_YES)         // replace any existing defn
                        };           

         System.out.print ("   Define QREMOTE          '"+sRemoteQtoFE+"'... ");
         
         // Use the agent to send the request
         responses = agent.send (CMQCFC.MQCMD_CREATE_Q, qremoteParameters);

         // check responses
         checkResponses(); 
         
         
         // --------------------------------------------------------------------------------------------------------------
         // Define Transmission Queue
         // --------------------------------------------------------------------------------------------------------------

         PCFParameter[] xmitqParameters =
                        {
                           new MQCFST (CMQC.MQCA_Q_NAME, sShortname),                  // queue name
                           new MQCFIN (CMQC.MQIA_Q_TYPE, CMQC.MQQT_LOCAL),             // qtype = local 
                           new MQCFIN (CMQC.MQIA_USAGE, CMQC.MQUS_TRANSMISSION),       // define as xmitq
                           new MQCFIN (CMQCFC.MQIACF_REPLACE, CMQCFC.MQRP_YES)         // replace any existing defn
                        };         
                        
         System.out.print ("   Define XMITQ            '"+sShortname+"'... ");
         
         // Use the agent to send the request         
         responses = agent.send (CMQCFC.MQCMD_CREATE_Q, xmitqParameters);

         // check responses
         checkResponses();                   
         
         
         // --------------------------------------------------------------------------------------------------------------               
         // Define Server Channel
         // --------------------------------------------------------------------------------------------------------------         
         
         String sServerChannelName =  sRegSysQMgr + "." + sShortname;
         
         PCFParameter[] serverChannelParameters = 
                        {
                           new MQCFST (CMQCFC.MQCACH_CHANNEL_NAME, sServerChannelName),  
                           new MQCFIN (CMQCFC.MQIACH_CHANNEL_TYPE, CMQXC.MQCHT_SERVER),
                           new MQCFST (CMQCFC.MQCACH_XMIT_Q_NAME, sShortname)            // F/E Qmgr name
                        };        

         System.out.print ("   Define server channel   '"+sServerChannelName+"'... ");

         // Use the agent to send the request
         responses = agent.send (CMQCFC.MQCMD_CREATE_CHANNEL, serverChannelParameters);

         // check responses
         checkResponses();                  
                  
      }
      catch (MQException mqe)
      {
         switch (mqe.reasonCode)
         {                                                                   
            case 2059 : System.out.println ("RC 2059. Local Queue Manager not started");
                        System.exit(0);                    
                                                                
            default   : System.out.println ("MQ exception caught:\n" + mqe);
         }   
      }
      catch (IOException ioe)
      {
         System.err.println (ioe);
      }
   System.exit(0);   
   }   


   public static void ResetChannel(String sChannelToReset)
   {
      try
      {   
         PCFAgent agent;
      
         // Connect a PCFAgent to the specified queue manager
         System.out.println("\nConnecting to queue manager " + sRegSysQMgr + "... ");
         agent = new PCFAgent (sRegSysQMgr);
         
         System.out.println ("Sending PCF requests ... ");
         
         // --------------------------------------------------------------------------------------------------------------         
         // Channel Reset
         // --------------------------------------------------------------------------------------------------------------         

         PCFParameter[] sChannelToResetParameters =
                        {
                           new MQCFST (CMQCFC.MQCACH_CHANNEL_NAME, sChannelToReset) 
                        };           

         System.out.print ("   Reset Channel           '"+sChannelToReset+"'... ");
         
         // Use the agent to send the request
         responses = agent.send (CMQCFC.MQCMD_RESET_CHANNEL, sChannelToResetParameters);

         // check responses
         checkResponses(); 
  
      }
      catch (MQException mqe)
      {
         switch (mqe.reasonCode)
         {                                                                   
            case 2059 : System.out.println ("RC 2059. Local Queue Manager not started");
                        System.exit(0);                    
                                                                
            default   : System.out.println ("MQ exception caught:\n" + mqe);
         }   
      }
      catch (IOException ioe)
      {
         System.err.println (ioe);
      }
   System.exit(0);   
   }     
           

   // ----------------------------------------------------------------------------------------------------------------
   // Method: checkResponses()
   // ----------------------------------------------------------------------------------------------------------------
   public static void checkResponses()
   {
      try
      {
         for (int i = 0; i < responses.length; i++)
         {
            cfh = new MQCFH (responses [i]);
         
            // Check the PCF header (MQCFH) in the response message

            switch (cfh.reason)
            {
               case 0    : System.out.println ("Created/Reset OK");      
                           break;                                                             
                                               
               case 4042 : System.out.println ("Channel already exists");
                           i++;    // to ignore PCF failed message
                           break;                               
                        
               default   : System.out.println ("PCF error:\n" + cfh);

               // Walk through the returned parameters describing the error

               for (int j = 0; j < cfh.parameterCount; j++)
               {
                  System.out.println (PCFParameter.nextParameter (responses [0]));
               }
            }
         }
      }
      catch (EOFException eofe)
      {
         ;
      }
      catch (IOException ioe)
      {
         System.err.println (ioe);
      }       
      catch (InstantiationException ie)
      {
         System.err.println (ie);
      }          
   }
}



